Package: UCSCXenaShiny
Title: Interactive Analysis of UCSC Xena Data
Version: 2.2.1
Authors@R: c(
    person("Shixiang", "Wang", , "w_shixiang@163.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-9855-7357")),
    person("Shensuo", "Li", , "lishensuo@163.com", role = "aut"),
    person("Yi", "Xiong", , "neuroxiong@openbiox.com", role = "aut",
           comment = c(ORCID = "0000-0002-4370-9824")),
    person("Longfei", "Zhao", , "longfei8533@126.com", role = "aut",
           comment = c(ORCID = "0000-0002-6277-0137")),
    person("Kai", "Gu", , "gukai1212@163.com", role = "aut",
           comment = c(ORCID = "0000-0002-0177-0774")),
    person("Yin", "Li", , "yinli18@fudan.edu.cn", role = "aut"),
    person("Fei", "Zhao", , "zhaofei415@mails.ucas.edu.cn", role = "aut")
  )
Maintainer: Shixiang Wang <w_shixiang@163.com>
Description: Provides functions and a Shiny application for downloading,
    analyzing and visualizing datasets from UCSC Xena
    (<http://xena.ucsc.edu/>), which is a collection of UCSC-hosted public
    databases such as TCGA, ICGC, TARGET, GTEx, CCLE, and others.
License: GPL (>= 3)
URL: https://github.com/openbiox/UCSCXenaShiny,
        https://openbiox.github.io/UCSCXenaShiny/
BugReports: https://github.com/openbiox/UCSCXenaShiny/issues
Depends: R (>= 4.1)
Imports: digest, dplyr (>= 0.8.3), ezcox, forcats, ggplot2 (>= 3.2.0),
        ggpubr (>= 0.2), httr, magrittr (>= 1.5), ppcor, psych, purrr,
        rlang, shiny (>= 1.3.2), stats, stringr, tibble (>= 2.1.3),
        tidyr, UCSCXenaTools, utils
Suggests: covr (>= 3.2.1), cowplot, DT (>= 0.5), funkyheatmap, furrr,
        future, ggrepel, ggstatsplot, glue, knitr, pacman, plotly,
        plyr, RColorBrewer (>= 1.1.2), readr, rmarkdown, Rtsne, scales,
        survival, survminer, testthat (>= 2.0.1), umap
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
RoxygenNote: 7.3.2
Packaged: 2026-05-05 12:09:52 UTC; wsx
Author: Shixiang Wang [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9855-7357>),
  Shensuo Li [aut],
  Yi Xiong [aut] (ORCID: <https://orcid.org/0000-0002-4370-9824>),
  Longfei Zhao [aut] (ORCID: <https://orcid.org/0000-0002-6277-0137>),
  Kai Gu [aut] (ORCID: <https://orcid.org/0000-0002-0177-0774>),
  Yin Li [aut],
  Fei Zhao [aut]
Repository: CRAN
Date/Publication: 2026-05-05 14:14:38 UTC
Built: R 4.6.0; ; 2026-05-05 17:34:55 UTC; unix
