# Generated by roxygen2: do not edit by hand

export("%>%")
export(.opt_pancan)
export(analyze_gene_drug_response_asso)
export(analyze_gene_drug_response_diff)
export(app_run)
export(app_run2)
export(available_hosts)
export(calc_pw_mut_cnv)
export(ezcor)
export(ezcor_batch)
export(ezcor_partial_cor)
export(get_ccle_cn_value)
export(get_ccle_gene_value)
export(get_ccle_mutation_status)
export(get_ccle_protein_value)
export(get_nonomics_value)
export(get_pancan_cn_value)
export(get_pancan_gene_value)
export(get_pancan_methylation_value)
export(get_pancan_miRNA_value)
export(get_pancan_mutation_status)
export(get_pancan_protein_value)
export(get_pancan_transcript_value)
export(get_pancan_value)
export(get_pcawg_APOBEC_mutagenesis_value)
export(get_pcawg_fusion_value)
export(get_pcawg_gene_value)
export(get_pcawg_miRNA_value)
export(get_pcawg_promoter_value)
export(keep_cat_cols)
export(load_data)
export(mol_quick_analysis)
export(query_molecule_value)
export(query_pancan_value)
export(query_tcga_group)
export(query_toil_value_df)
export(tcga_surv_get)
export(tcga_surv_plot)
export(vis_ccle_gene_cor)
export(vis_ccle_tpm)
export(vis_dim_dist)
export(vis_gene_TIL_cor)
export(vis_gene_cor)
export(vis_gene_cor_cancer)
export(vis_gene_cross_omics)
export(vis_gene_drug_response_asso)
export(vis_gene_drug_response_diff)
export(vis_gene_immune_cor)
export(vis_gene_msi_cor)
export(vis_gene_pw_cor)
export(vis_gene_stemness_cor)
export(vis_gene_tmb_cor)
export(vis_identifier_cor)
export(vis_identifier_dim_dist)
export(vis_identifier_grp_comparison)
export(vis_identifier_grp_surv)
export(vis_identifier_multi_cor)
export(vis_pancan_anatomy)
export(vis_pathway_cross_omics)
export(vis_pcawg_dist)
export(vis_pcawg_gene_cor)
export(vis_pcawg_unicox_tree)
export(vis_toil_Mut)
export(vis_toil_Mut_cancer)
export(vis_toil_TvsN)
export(vis_toil_TvsN_cancer)
export(vis_unicox_tree)
import(dplyr)
import(forcats)
import(ggplot2)
import(ppcor)
import(purrr)
import(tibble)
importFrom(magrittr,"%>%")
importFrom(psych,corr.test)
importFrom(shiny,shinyAppFile)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,tail)
