## Avoid CRAN checking NOTEs

utils::globalVariables(
  c(
    "XenaHostNames",
    "DataSubtype",
    "XenaDatasets",
    ".",
    "Type",
    "group",
    ".env",
    "icgc_specimen_id",
    "dcc_project_code",
    "OS",
    "OS.time",
    "pval",
    "cor"
  )
)
