#' @title Xena Shiny App
#' @description A Shiny App for UCSC Xena Data Hubs. See <https://github.com/openbiox/UCSCXenaShiny> for details.
#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import ggplot2
#' @importFrom psych corr.test
#' @importFrom utils download.file head install.packages tail
#' @importFrom stats prcomp predict quantile na.omit
## usethis namespace: end
NULL
