- <font size="4">In this step, you can query multiple identifiers of tumor data under one specific "**Data type**"--"**Data subtype**;</font> 
- <font size="4">Generally, the following 3 way can be performed to select batch IDs;</font></font>

> 1. In the "**Selection**" way, choose multiple IDs one by one;
> 2. In the "**All**" way, directly choose all IDs under the data subtype. For molecular profile, you need further choose one pathway to limit the ID size;
> 3. In the "**File**" way, choose IDs by uploading an one-column file. You can download the example data to check the format requirements.

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_get_batch_data.png" alt="helper_get_batch_data"   width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 





