- <font size="4"> In this step, you can upload custom TCGA/PCAWG/CCLE sample data for joint analysis. The example data can be downloded to check the relevant format requirements;</font> 
- <font size="4"> The uploaded data will be placed with **Custom metadata** (Data subtype) of Phenotype data (Data type) for analysis;</font>


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_costom_metadata.png" alt="helper_costom_metadata"  width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 