#### [PCAWG project types](https://dcc.icgc.org/releases/release_28/Projects)
<br>

| Study Abbreviation | Study Name                                                 |
| ------------------ | ---------------------------------------------------------- |
| BLCA-US            | Bladder Urothelial Cancer - TCGA, US                       |
| BRCA-US            | Breast Cancer - TCGA, US                                   |
| CESC-US            | Cervical Squamous Cell Carcinoma - TCGA, US                |
| CLLE-ES            | Chronic Lymphocytic Leukemia - ES                          |
| COAD-US            | Colon Adenocarcinoma - TCGA, US                            |
| DLBC-US            | Lymphoid Neoplasm Diffuse Large B-cell Lymphoma - TCGA, US |
| ESAD-UK            | Esophageal Adenocarcinoma - UK                             |
| GBM-US             | Brain Glioblastoma Multiforme - TCGA, US                   |
| HNSC-US            | Head and Neck Squamous Cell Carcinoma - TCGA, US           |
| KICH-US            | Kidney Chromophobe - TCGA, US                              |
| KIRC-US            | Kidney Renal Clear Cell Carcinoma - TCGA, US               |
| KIRP-US            | Kidney Renal Papillary Cell Carcinoma - TCGA, US           |
| LAML-US            | Acute Myeloid Leukemia - TCGA, US                          |
| LGG-US             | Brain Lower Grade Glioma - TCGA, US                        |
| LIHC-US            | Liver Hepatocellular carcinoma - TCGA, US                  |
| LIRI-JP            | Liver Cancer - RIKEN, JP                                   |
| LUAD-US            | Lung Adenocarcinoma - TCGA, US                             |
| LUSC-US            | Lung Squamous Cell Carcinoma - TCGA, US                    |
| MALY-DE            | Malignant Lymphoma - DE                                    |
| OV-AU              | Ovarian Cancer - AU                                        |
| OV-US              | Ovarian Serous Cystadenocarcinoma - TCGA, US               |
| PACA-AU            | Pancreatic Cancer - AU                                     |
| PRAD-US            | Prostate Adenocarcinoma - TCGA, US                         |
| READ-US            | Rectum Adenocarcinoma - TCGA, US                           |
| RECA-EU            | Renal Cell Cancer - EU/FR                                  |
| SARC-US            | Sarcoma - TCGA, US                                         |
| SKCM-US            | Skin Cutaneous melanoma - TCGA, US                         |
| STAD-US            | Gastric Adenocarcinoma - TCGA, US                          |
| THCA-US            | Head and Neck Thyroid Carcinoma - TCGA, US                 |
| UCEC-US            | Uterine Corpus Endometrial Carcinoma- TCGA, US             |