
- <font size="4"> In this step, you can check which UCSC Xena hub and dataset each type of **Molecular profile** comes from for downstream analysis. You can click on the provided hub and dataset links at the bottom to learn more details about them；</font>
- <font size="4"> For some data types, there are **multiple datasets** available based on factors such as standardization methods, sequencing platforms, and others； </font>
- <font size="4"> Noteworthily, we provide more personlized options for **DNA methylation** type (Only TCGA); </font>

>1. Select one methylation platform;
>2. Select one gene;
>3. Select CpG sites through chromosomal coordinates.  By default, it is the range of all sites for that gene;
>4. Further manually select CpG sites based on above coordinates. By default, all sites will be selected
>5. Select the aggregation method for calculating gene-level methylation. By default, the mean of the selected CpG sites will be calculated.

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_data_origin.png" alt="helper_data_origin"  width="450" />
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 