- In this step, you can choose one tumor data to divide the samples into two groups; - First, select an identifier and observe its distribution among samples; - Then, set the ranges for the two groups as the grouping criterion; > 1. For **character** class, you should assign one or multiple categories for either group; > 2. For **numeric** class, you can set two ranges by the absolute and percentile intervals.

--- - See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/).