- <font size="4">In this step, you can choose one tumor data to divide the samples into two groups;</font> 
- <font size="4">First, select an identifier and observe its distribution among samples;</font> 
- <font size="4">Then, set the ranges for the two groups as the grouping criterion;</font> 

> 1. For **character** class, you should assign one or multiple categories for either group;
> 2. For **numeric** class, you can set two ranges by the absolute and percentile intervals.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_set_groups.png" alt="helper_set_groups"  width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 