<center> <h2> UCSCXenaShiny v2 </h2> </center>

- <font size="5">Shensuo Li, Yuzhong Peng, Minjun Chen, Yankun Zhao, Yi Xiong, Jianfeng Li, Peng Luo, Haitao Wang, Fei Zhao, Qi Zhao, Yanru Cui, Sujun Chen, Jian-Guo Zhou, Shixiang Wang,  Facilitating integrative and personalized oncology omics analysis with UCSCXenaShiny, Communications Biology, 1200 (2024),  https://doi.org/10.1038/s42003-024-06891-2</font>

- <font size="5">PMID: [39341906](https://pubmed.ncbi.nlm.nih.gov/39341906/)</font>
- <font size="5">Bibtex format: </font>

```
@ARTICLE{Li2024-yd,
  title    = "Facilitating integrative and personalized oncology omics analysis with UCSCXenaShiny",
  author   = "Li, Shensuo and Peng, Yuzhong and Chen, Minjun and Zhao, Yankun and Xiong, Yi and Li, Jianfeng and Luo, Peng and Wang, Haitao and Zhao, Fei and Zhao, Qi and Cui, Yanru and Chen, Sujun and Zhou, Jian-Guo and Wang, Shixiang",
  abstract = "The continuous generation of multi-omics and phenotype data is propelling advancements in precision oncology. UCSCXenaShiny was developed as an interactive tool for exploring thousands of cancer datasets available on UCSC Xena. However, its capacity for comprehensive and personalized pan-cancer data analysis is being challenged by the growing demands. Here, we introduce UCSCXenaShiny v2, a milestone update through a variety of improvements. Firstly, by integrating multidimensional data and implementing adaptable sample settings, we create a suite of robust TPC (TCGA, PCAWG, CCLE) analysis pipelines. These pipelines empower users to conduct in-depth analyses of correlation, comparison, and survival in three modes: Individual, Pan-cancer and Batch screen. Additionally, the tool includes download interfaces that enable users to access diverse data and outcomes, several features also facilitate the joint analysis of drug sensitivity and multi-omics of cancer cell lines. UCSCXenaShiny v2 is an open-source R package and a web application, freely accessible at https://github.com/openbiox/UCSCXenaShiny.",
  journal  = "Commun. Biol.",
  volume   =  7,
  number   =  1,
  pages    = "1200",
  month    =  sep,
  year     =  2024,
  language = "en"
}
```









<br>

<center> <h2> UCSCXenaShiny v1 </h2> </center>

- <font size="5">Shixiang Wang<sup>\#</sup>, Yi Xiong<sup>\#</sup>, Longfei Zhao<sup>\#</sup>, Kai Gu<sup>\#</sup>, Yin Li, Fei Zhao, Jianfeng Li, Mingjie Wang, Haitao Wang, Ziyu Tao, Tao Wu, Yichao Zheng, Xuejun Li, Xue-Song Liu, UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data, Bioinformatics, 2021;, btab561, https://doi.org/10.1093/bioinformatics/btab561.</font>



- <font size="5">PubMed ID: [34323947](https://pubmed.ncbi.nlm.nih.gov/34323947/)</font>
- <font size="5">Bibtex format:</font>

```
@article{10.1093/bioinformatics/btab561,
    author = {Wang, Shixiang and Xiong, Yi and Zhao, Longfei and Gu, Kai and Li, Yin and Zhao, Fei and Li, Jianfeng and Wang, Mingjie and Wang, Haitao and Tao, Ziyu and Wu, Tao and Zheng, Yichao and Li, Xuejun and Liu, Xue-Song},
    title = "{UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data}",
    journal = {Bioinformatics},
    year = {2021},
    month = {07},
    abstract = "{UCSC Xena platform provides huge amounts of processed cancer omics data from large cancer research projects (e.g. TCGA, CCLE and PCAWG) or individual research groups and enables unprecedented research opportunities. However, a graphical user interface (GUI) based tool for interactively analyzing UCSC Xena data and generating elegant plots is still lacking, especially for cancer researchers and clinicians with limited programming experience. Here, we present UCSCXenaShiny, an R Shiny package for quickly searching, downloading, exploring, analyzing and visualizing data from UCSC Xena data hubs. This tool could effectively promote the practical use of public data, and can serve as an important complement to the current Xena genomics explorer.UCSCXenaShiny is an open source R package under GPLv3 license and it is freely available at https://github.com/openbiox/UCSCXenaShiny or https://cran.r-project.org/package=UCSCXenaShiny. The docker image is available at https://hub.docker.com/r/shixiangwang/ucscxenashiny.Supplementary data are available at Bioinformatics online.}",
    issn = {1367-4803},
    doi = {10.1093/bioinformatics/btab561},
    url = {https://doi.org/10.1093/bioinformatics/btab561},
    note = {btab561},
    eprint = {https://academic.oup.com/bioinformatics/advance-article-pdf/doi/10.1093/bioinformatics/btab561/39456169/btab561.pdf},
}
```
<br>

<center> <h2> UCSC Xena </h2> </center>


<font size="5">When you use UCSC Xena data, you should also cite it with:</font>

- <font size="5">Goldman, M. J., Craft, B., Hastie, M., Repečka, K., McDade, F., Kamath, A., ... & Haussler, D. (2020). Visualizing and interpreting cancer genomics data via the Xena platform. Nature biotechnology, 38(6), 675-678. https://doi:10.1038/s41587-020-0546-8</font>

- <font size="5">PubMed ID: [32444850](https://pubmed.ncbi.nlm.nih.gov/32444850/)</font>
- <font size="5">Bibtex format:</font>

```
@article{goldman2020visualizing,
  title={Visualizing and interpreting cancer genomics data via the Xena platform},
  author={Goldman, Mary J and Craft, Brian and Hastie, Mim and Repe{\v{c}}ka, Kristupas and McDade, Fran and Kamath, Akhil and Banerjee, Ayan and Luo, Yunhai and Rogers, Dave and Brooks, Angela N and others},
  journal={Nature biotechnology},
  volume={38},
  number={6},
  pages={675--678},
  year={2020},
  publisher={Nature Publishing Group}
}
```

