

- <font size="4">In this step, you can query specific tumor data according to the "**Data type**"--"**Data subtype**"--"**Identifier**" search sequence;</font> 
- <font size="4">As "**Molecular profile**" data type needs to be obtained from the UCSC Xena site, the caching time depends on the network conditions;</font> 


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_get_one_data.png" alt="helper_get_one_data" width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 