<center>
<h1>TPC pipelines</h1>
</center>


<font size="5">Here, we introduced a series of personalized TPC pipelines for comprehensive and precise TPC molecule exploration with various operations.  In general, 9 panels for common analytical scenario are available, as follows:</font>


<center>
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_intro_personal_pips_1.png" alt="helper_intro_personal_pips_1"  style="zoom:50%";/>
</center>


<font size="5">Generally, each analytical pipeline comprises of three main subpanels (S1, S2, S3).</font>

- <font size="5">In S1 subpanel, you can prepare and select the expected data and sample ranges;</font>
- <font size="5">In S2 subpanel, you can fetch the specific tumor data for downstream analysis;</font>
- <font size="5">In S3 subpanel, you can execute the analysis, visualize and download results. </font>



<center>
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_intro_personal_pips_2.png" alt="helper_intro_personal_pips_2" style="zoom:80%;" />
</center>


<br></br>

<br></br>



> <font size="5">Tips: The menu of `TPC modules` enable specific and quick  TPC molecular analysis, such as tumor and tomor comparison, molecule and molecule correlation.</font>
