<font size="5">In this step, you should perform two types of operations</font>





<font size="4">**In first step**, you can query multiple identifiers of tumor data under one specific "**Data type**"--"**Data subtype**;</font> 

- <font size="4">Generally, the following 3 way can be performed to select and cache batch IDs;</font></font>

> 1. In the "**Selection**" way, choose multiple IDs one by one;
> 2. In the "**All**" way, directly choose all IDs under the data subtype. For molecular profile, you need further choose one pathway to limit the ID size;
> 3. In the "**File**" way, choose IDs by uploading an one-column file. You can download the example data to check the format requirements.

<font size="4">**In second step**, you should use above tumor data to divide the samples into two groups;</font> 

- <font size="4">Set the ranges for the two groups as the grouping criterion;</font> 
> 1. For **character** class, you should assign one or multiple categories for either group;
> 2. For **numeric** class, you can set two ranges by the absolute and percentile intervals.


<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_get_batch_data_set_groups.png" alt="helper_get_batch_data_set_groups" width="700"/>
<p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 

