- <font size="4">In **S3** Step, **batch survival analysis** can now be performed after data preparation;</font>
- <font size="4">Firstly, you can adjust the default analysis and visualization parameters, respectively;</font>
- <font size="4">Noteworthily, you can try whether to use **initial data** before grouping if the data type is **numerical**;</font>

> 1. For log-rank test, it will decide the optimal cutoff;
> 2. For univariate cox regression, it will directly analyze based on the numerical data. 

- <font size="4">Then, you will obtain the table arranged by p.value  after clicking the "**Run**" button;</font>
- <font size="4">Finally, you can download the 2 types of results.</font>

> 1. "**Raw data(.csv)**": queried data for X-axis and Y-axis;
> 2. "**Analyzed data(.csv)**": detailed correlation result.




<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_analyze_sur_3.png" alt="helper_analyze_sur_3" width="700"/>
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 