<center> <h2> Citation </h2> </center>

Please cite the following article when you used **UCSCXenaShiny** in your study:

Shixiang Wang<sup>\#</sup>, Yi Xiong<sup>\#</sup>, Longfei Zhao<sup>\#</sup>, Kai Gu<sup>\#</sup>, Yin Li, Fei Zhao, Jianfeng Li, Mingjie Wang, Haitao Wang, Ziyu Tao, Tao Wu, Yichao Zheng, Xuejun Li, Xue-Song Liu, UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data, Bioinformatics, 2021;, btab561, https://doi.org/10.1093/bioinformatics/btab561.

Bibtex format:

```
@article{10.1093/bioinformatics/btab561,
    author = {Wang, Shixiang and Xiong, Yi and Zhao, Longfei and Gu, Kai and Li, Yin and Zhao, Fei and Li, Jianfeng and Wang, Mingjie and Wang, Haitao and Tao, Ziyu and Wu, Tao and Zheng, Yichao and Li, Xuejun and Liu, Xue-Song},
    title = "{UCSCXenaShiny: An R/CRAN Package for Interactive Analysis of UCSC Xena Data}",
    journal = {Bioinformatics},
    year = {2021},
    month = {07},
    abstract = "{UCSC Xena platform provides huge amounts of processed cancer omics data from large cancer research projects (e.g. TCGA, CCLE and PCAWG) or individual research groups and enables unprecedented research opportunities. However, a graphical user interface (GUI) based tool for interactively analyzing UCSC Xena data and generating elegant plots is still lacking, especially for cancer researchers and clinicians with limited programming experience. Here, we present UCSCXenaShiny, an R Shiny package for quickly searching, downloading, exploring, analyzing and visualizing data from UCSC Xena data hubs. This tool could effectively promote the practical use of public data, and can serve as an important complement to the current Xena genomics explorer.UCSCXenaShiny is an open source R package under GPLv3 license and it is freely available at https://github.com/openbiox/UCSCXenaShiny or https://cran.r-project.org/package=UCSCXenaShiny. The docker image is available at https://hub.docker.com/r/shixiangwang/ucscxenashiny.Supplementary data are available at Bioinformatics online.}",
    issn = {1367-4803},
    doi = {10.1093/bioinformatics/btab561},
    url = {https://doi.org/10.1093/bioinformatics/btab561},
    note = {btab561},
    eprint = {https://academic.oup.com/bioinformatics/advance-article-pdf/doi/10.1093/bioinformatics/btab561/39456169/btab561.pdf},
}
```

When you use UCSC Xena data, you should also cite it with:

Goldman, M. J., Craft, B., Hastie, M., Repečka, K., McDade, F., Kamath, A., ... & Haussler, D. (2020). Visualizing and interpreting cancer genomics data via the Xena platform. Nature biotechnology, 38(6), 675-678.

Bibtex format:

```
@article{goldman2020visualizing,
  title={Visualizing and interpreting cancer genomics data via the Xena platform},
  author={Goldman, Mary J and Craft, Brian and Hastie, Mim and Repe{\v{c}}ka, Kristupas and McDade, Fran and Kamath, Akhil and Banerjee, Ayan and Luo, Yunhai and Rogers, Dave and Brooks, Angela N and others},
  journal={Nature biotechnology},
  volume={38},
  number={6},
  pages={675--678},
  year={2020},
  publisher={Nature Publishing Group}
}
```

<center> <h2> Contact </h2> </center>

### Bug report or feature request

If you find any bug or have any feature to request, please report it at GitHub <https://github.com/openbiox/UCSCXenaShiny/issues> or email to me <mailto:w_shixiang@163.com>.

If you love our work, please give use a star at <https://github.com/openbiox/UCSCXenaShiny>.

### Discussion

To discuss ideas or suggestions, you can contact any of us via information shown in "Developers" page.
