- <font size="4">In this step, you can set custom signature based on molecular profile of TCGA/PCAWG/CCLE sample data for joint analysis;</font>
- <font size="4">You can (1) set the basic signature information and (2) molecular formula, (3) calculate the signature value and add into **Custom metadata** (Data subtype) of Phenotype data (Data type) for analysis;</font>

<p align="center">
<img src="https://ucscxenashiny-1301043367.cos.ap-shanghai.myqcloud.com/Shiny-figures/helper_add_signature.png" alt="helper_add_signature"    width="700"  />
</p>

---

- <font size="4"> See more details in [UCSCXenaShiny v2 book](https://lishensuo.github.io/UCSCXenaShiny_Book/). </font> 
