# TPC pipelines

Here, we introduced a series of personalized TPC pipelines for comprehensive and precise TPC molecule exploration with various operations. In general, 9 panels for common analytical scenario are available, as follows:
Generally, each analytical pipeline comprises of three main subpanels (S1, S2, S3). - In S1 subpanel, you can prepare and select the expected data and sample ranges; - In S2 subpanel, you can fetch the specific tumor data for downstream analysis; - In S3 subpanel, you can execute the analysis, visualize and download results.
> Tips: The menu of `TPC modules` enable specific and quick TPC molecular analysis, such as tumor and tomor comparison, molecule and molecule correlation.