% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pancan_value.R
\name{available_hosts}
\alias{available_hosts}
\title{Show Available Hosts}
\usage{
available_hosts()
}
\value{
hosts
}
\description{
Show Available Hosts
}
\examples{
available_hosts()
}
