% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_subtypes}
\alias{tcga_subtypes}
\title{TCGA Subtype Data}
\format{
A \link{data.frame}
}
\source{
UCSC Xena.
}
\description{
TCGA Subtype Data
}
\examples{
data("tcga_subtypes")
}
