% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tcga_purity}
\alias{tcga_purity}
\title{TCGA: Purity Data}
\format{
A \link{data.frame}
}
\source{
\url{https://www.nature.com/articles/ncomms9971#Sec14}
}
\description{
TCGA: Purity Data
}
\examples{
data("tcga_purity")
}
