% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{TCGA.organ}
\alias{TCGA.organ}
\title{TCGA: Organ Data}
\format{
A \link{data.frame}
}
\description{
TCGA: Organ Data
}
\examples{
data("TCGA.organ")
}
