% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pcawg_purity}
\alias{pcawg_purity}
\title{Purity Data of PCAWG}
\format{
A \code{data.frame}
}
\source{
UCSC Xena.
}
\description{
Purity Data of PCAWG
}
\examples{
data("pcawg_purity")
}
